
package de.codedo.conflicteditor;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.hasSize;

import java.io.IOException;
import java.util.List;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;
import org.junit.Test;

public class DocumentMatcherTestCase extends Object
{
    private ObjectMapper _mapper = new ObjectMapper();

    @Test
    public void equalDocumentsShouldProduceNoDifferences() throws Exception
    {
        JsonNode node = parse("{ 'key' : 'value' }");
        List<Difference> diffs = new DocumentMatcher(node, node).compare();
        assertThat(diffs, hasSize(0));
    }

    @Test
    public void differentValuesShouldProduceOneDifferencePerKey() throws Exception
    {
        JsonNode original = parse("{ 'key' : 'value' }");
        JsonNode conflict = parse("{ 'key' : 'other-value' }");
        List<Difference> diffs = new DocumentMatcher(original, conflict).compare();
        assertThat(diffs, hasSize(1));

        Difference actualDifference = diffs.get(0);
        Difference expectedDifference = new Difference("key", "value", "other-value");
        assertThat(actualDifference, is(expectedDifference));
    }

    @Test
    public void valuePresentOnlyInOriginalShouldProduceDifference() throws Exception
    {
        JsonNode original = parse("{ 'key' : 'value', 'only-here' : 'value' }");
        JsonNode conflict = parse("{ 'key' : 'value' }");
        List<Difference> diffs = new DocumentMatcher(original, conflict).compare();
        assertThat(diffs, hasSize(1));

        Difference actualDifference = diffs.get(0);
        Difference expectedDifference = new Difference("only-here", "value", null);
        assertThat(actualDifference, is(expectedDifference));
    }

    @Test
    public void valuePresentOnlyInConflictShouldProduceDifference() throws Exception
    {
        JsonNode original = parse("{ 'key' : 'value' }");
        JsonNode conflict = parse("{ 'key' : 'value', 'only-here' : 'value' }");
        List<Difference> diffs = new DocumentMatcher(original, conflict).compare();
        assertThat(diffs, hasSize(1));

        Difference actualDifference = diffs.get(0);
        Difference expectedDifference = new Difference("only-here", null, "value");
        assertThat(actualDifference, is(expectedDifference));
    }

    private JsonNode parse(String input) throws IOException
    {
        // Jackson cannot parse JSON that uses single quotes ...
        String convertedInput = input.replace("'", "\"");
        return _mapper.readTree(convertedInput);
    }
}
